﻿itf2img
====
筑波大学超小型人工衛星ITF-2のための画像ダウンリンクデータ作成・解析ツール

特徴
======
 * テキストファイルからPNG画像を復元する
 * PNG画像からテキストファイルを作成する
 * 省サイズな[画像フォーマット](ITF-2_image_format.pdf)を使用

使い方
======

受信したデータをデコードする(16進数表記の場合)
----
ITF-2からダウンリンクされるF2Dのパケットを[direwolf](https://github.com/wb2osz/direwolf)などのソフトウェアを用いてデコードした際、データは以下のような16進数表記で表示される。<br>

494D30523907070410FF54545454FFFFFF5454FFFFFF54FF9F6F3F3F6F9F6F3F9F6F9F3F9F6F9F3F9F6FFFA400C0C0C0FFFF00FF00FFFFFFFFE023456789ABCDF1F1F11F1F1F11FFF1FFF11F1F1F11F1FFF0<br>

これは[画像ダウンリンクフォーマット](ITF-2_image_format.pdf)に従った画像データの例である。<br>
先頭の3Bytes分は`494D30`であり、これはascii文字列の`IM0`である。<br>

`494D30`以降の文字列をテキストファイルに`hex.txt`という名前で保存する。<br>
名前は`hex.txt`でない他のものでも構わない。<br>

このデータを画像に復元するために以下のコマンドを実行する<br>

```
./itf2img h2p -o img.png hex.txt
```

`h2p`は、16進数表記(Hexadecimal)のテキストファイルを読み込み、PNG画像を書き出すitf2imgの組み込みコマンドである。<br>
`-o`オプションは画像ファイル名を指定するもので、省略された場合には、デフォルトの`a.png`という名前になる。<br>

出力された`img.png`は以下の画像と同じになる。<br>
<img src="./sample/img0_4bit.png"><br>

受信したデータをデコードする(10進数表記の場合)
----
以下のようなデータを持つファイル`dec.txt`があるとする。<br>

73 77 48 82 57 7 7 4 16 255 84 84 84 84 255 255 255 84 84 255 255 255 84 255 159 111 63 63 111 159 111 63 159 111 159 63 159 111 159 63 159 111 255 164 0 192 192 192 255 255 0 255 0 255 255 255 255 224 35 69 103 137 171 205 241 241 241 31 31 31 17 255 241 255 241 31 31 31 17 241 255 240 <br>

先頭の3Bytes分は`73 77 48`であり、これはAscii文字列の`IM0`である。<br>
16進数表記の時とは違い、各データはスペース1文字で区切られていることが必要である。<br>

このデータを画像に復元するために以下のコマンドを実行する。<br>

```
./itf2img d2p -o img.png hex.txt
```

`d2p`は、10進数表記(Decimal)のテキストファイルを読み込み、PNG画像を書き出すitf2imgの組み込みコマンドである。<br>
`-o`オプションは16進数の場合と同様に出力ファイル名を指定するものである。<br>
出力された`img.png`は以下の画像と同じになる。<br>
<img src="./sample/img0_4bit.png"><br>

画像をITF-2コマンド形式ファイルに変換する場合
----
この項目は筑波大学「結」プロジェクトのITF-2運用メンバーのためのものである。<br>

画像ダウンリンクには、ダウンリンクフォーマット番号40の「A系統長文メッセージ0」を使用する。<br>
このフォーマットはF2Dによるダウンリンクにのみ対応するものである。<br>
衛星の仕様上、F2Dで自由に使用できるデータサイズは、ヘッダ部分を除いた171Bytesである。<br>
16進数表記のasciiに変換すると、イラストで使えるデータサイズはその半分の85Bytesとなる。<br>

よって、画像データをアップリンクするには、生のPNG画像のデータでは大きすぎるため、これを独自の小サイズな画像フォーマットに変換する必要がある。<br>
このフォーマットに関しては同包の[ITF-2\_image\_format.pdf](-2_image_format.pdf)に詳細を記す.<br>

画像[sample/img0\_4bit.png](sample/img0_4bit.png)をコマンドファイルに変換するためには、以下のコマンドを実行する。<br>

```
./itf2img d2c -o cmd.txt img.png
```

`p2c`は、PNG画像を読み込み、ITF-2のアップリンクコマンドファイルを書き出すitf2imgの組み込みコマンドである。<br>
`-o`オプションは16進数の場合と同様に出力ファイル名を指定するものである。<br>
小サイズ化した後のデータサイズが85Bytesを超えてしまうとエラーとなり、変換に失敗するので注意すること。<br>
変換に成功すると、以下の内容をもつテキストファイル`cmd.txt`が得られる。<br>

"CMDS","17","12","19","12","51","25","DLD","164","52","57","52","61","51","48","53","50","51","57","48","55","48","55","48","52","49","48","63","63","53","52","53","52","53","52","53","52","63","63","63","63","63","63","53","52","53","52","63","63","63","63","63","63","53","52","63","63","57","63","54","63","51","63","51","63","54","63","57","63","54","63","51","63","57","63","54","63","57","63","51","63","57","63","54","63","57","63","51","63","57","63","54","63","63","63","58","52","48","48","60","48","60","48","60","48","63","63","63","63","48","48","63","63","48","48","63","63","63","63","63","63","63","63","62","48","50","51","52","53","54","55","56","57","58","59","60","61","63","49","63","49","63","49","49","63","49","63","49","63","49","49","63","63","63","49","63","63","63","49","49","63","49","63","49","63","49","49","63","49","63","63","63","48",<br>

生成されたこのアップリンクコマンドコマンドは、他のアップリンクコマンドと同様に地上局のdirewolfクライアントソフトウェア(TX系統)のアップリンクコマンドファイル保存ディレクトリに保存し、`CMD.INI`を編集して`cmd.txt`の保存先を登録することで使用できるようになる。<br>

組み込みコマンド、コマンドラインオプション
====

構文<br>
----

```
./itf2img [--version] [--help] [-verbose] <command> [<args>] <path>
```

コマンドラインオプション `[???]`
----
 * `--version` ヘルプを表示して終了する
 * `--help` ヘルプを表示して終了する
 * `-verbose` `-v` 詳細な画面出力を有効にする

組み込みコマンド `<command>`
----
 * `s` PNG画像の小サイズ化した後のデータサイズを表示する
 * `i` PNG画像の小サイズ化前後の表示詳細な情報を表示する
 * `h2p` 16進数表記テキストファイルをPNG画像に変換する
 * `d2p` 10進数表記テキストファイルをPNG画像に変換する
 * `p2h` PNG画像を16進数表記テキストファイルに変換する
 * `p2d` PNG画像を10進数表記テキストファイルに変換する
 * `p2c` PNG画像をITF-2のコマンドファイルに変換する

組み込みコマンドのオプション `[<args>]`
----
 * `-o <path>` 出力ファイル名を指定する (`h2p`, `d2p`, `p2h`, `p2d`, `p2c`コマンドで有効)

入力ファイルの指定 `<path>`
----
入力ファイルへを指定する。<br>

インストール方法
======
フォルダを適当なディレクトリに解凍すること<br>
アンインストールはフォルダごと削除すること<br>

依存ライブラリ
======
 * [zlib 1.2.X](http://www.zlib.net/)
 * [libpng 1.6.X](http://www.libpng.org/pub/png/libpng.html)
 * [Windows SDK](https://developer.microsoft.com/ja-jp/windows/downloads/windows-8-1-sdk) (windowsのみ)

ビルド方法
======
UNIXの場合<br>
----
事前にlibpngとzlibをインストールした後、以下のコマンドを実行。<br>

```
make -f makefile_unix.mk && install itf2img.out /usr/local/bin
```

Windowsの場合<br>
----
windowsでビルドする場合にはCygwinやMSYS2などのターミナルが必要。<br>
事前にSDKに必要なパスを通しておく。<br>
[makefile\_win32.mk](./makefile_win32.mk)のコンパイラ・リンカのパスは適切に書き換えること<br>

```
nmake /f makefile_win32.mk DEBUG=0
```

ライセンス
======
MITライセンスで配布する。<br>
[http://opensource.org/licenses/mit-license.php](http://opensource.org/licenses/mit-license.php)
